;; Generated package description from geiser-mit.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "geiser-mit" "0.15.0.20260727.3" "MIT/GNU Scheme's implementation of the geiser protocols" '((emacs "24.4") (geiser "0.18")) :commit "7ea6ae00331c44470957e0b90c80adf7a6b6565b" :authors '(("Peter" . "craven@gmx.net")) :maintainer '("Jose A Ortega Ruiz" . "jao@gnu.org") :keywords '("languages" "mit" "scheme" "geiser") :url "https://gitlab.com/emacs-geiser/mit")
